@@ -7,7 +7,7 @@
 // Copyright 2006-2008 Julien Rebetez <julienr@svn.gnome.org>
 // Copyright 2007-2012, 2015, 2016 Albert Astals Cid <aacid@kde.org>
 // Copyright 2007-2008, 2011 Carlos Garcia Campos <carlosgc@gnome.org>
-// Copyright 2007, 2013 Adrian Johnson <ajohnson@redneon.com>
+// Copyright 2007, 2013, 2016 Adrian Johnson <ajohnson@redneon.com>
 // Copyright 2007 Iñigo Martínez <inigomartinez@gmail.com>
 // Copyright 2008, 2011 Pino Toscano <pino@kde.org>
 // Copyright 2008 Michael Vrable <mvrable@cs.ucsd.edu>
@@ -1421,7 +1421,7 @@ void FormFieldSignature::parseInfo()
   sig_dict.dictLookup("M", &time_of_signing);
   if (time_of_signing.isString()) {
     GooString *time_str = time_of_signing.getString();
-    signature_info->setSigningTime(pdfTimeToInteger(time_str)); // Put this information directly in SignatureInfo object
+    signature_info->setSigningTime(dateStringToTime(time_str)); // Put this information directly in SignatureInfo object
     time_of_signing.free();
   }
 
