@@ -14,13 +14,14 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2005 Brad Hards <bradh@frogmouth.net>
-// Copyright (C) 2006, 2008, 2010-2012 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2006, 2008, 2010-2013 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2007-2008 Julien Rebetez <julienr@svn.gnome.org>
 // Copyright (C) 2007 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2010 Ilya Gorenbein <igorenbein@finjan.com>
 // Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
-// Copyright (C) 2012 Thomas Freitag <Thomas.Freitag@kabelmail.de>
-// Copyright (C) 2012 Fabio D'Urso <fabiodurso@hotmail.it>
+// Copyright (C) 2012, 2013, 2016 Thomas Freitag <Thomas.Freitag@kabelmail.de>
+// Copyright (C) 2012, 2013 Fabio D'Urso <fabiodurso@hotmail.it>
+// Copyright (C) 2013 Adrian Johnson <ajohnson@redneon.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -34,8 +35,11 @@
 #pragma interface
 #endif
 
+#include "poppler-config.h"
 #include "goo/gtypes.h"
+#include "goo/GooMutex.h"
 #include "Object.h"
+#include "Stream.h"
 
 #include <vector>
 
@@ -56,11 +60,34 @@ enum XRefEntryType {
 };
 
 struct XRefEntry {
-  Guint offset;
+  Goffset offset;
   int gen;
   XRefEntryType type;
-  bool updated;
+  int flags;
   Object obj; //if this entry was updated, obj will contains the updated object
+
+  enum Flag {
+    // Regular flags
+    Updated,     // Entry was modified
+
+    // Special flags -- available only after xref->scanSpecialFlags() is run
+    Unencrypted, // Entry is stored in unencrypted form (meaningless in unencrypted documents)
+    DontRewrite  // Entry must not be written back in case of full rewrite
+  };
+
+  inline GBool getFlag(Flag flag) {
+    const int mask = (1 << (int)flag);
+    return (flags & mask) != 0;
+  }
+
+  inline void setFlag(Flag flag, GBool value) {
+    const int mask = (1 << (int)flag);
+    if (value) {
+      flags |= mask;
+    } else {
+      flags &= ~mask;
+    }
+  }
 };
 
 class XRef {
@@ -71,11 +98,14 @@ public:
   // Constructor, create an empty XRef but with info dict, used for PDF writing
   XRef(Object *trailerDictA);
   // Constructor.  Read xref table from stream.
-  XRef(BaseStream *strA, Guint pos, Guint mainXRefEntriesOffsetA = 0, GBool *wasReconstructed = NULL, GBool reconstruct = false);
+  XRef(BaseStream *strA, Goffset pos, Goffset mainXRefEntriesOffsetA = 0, GBool *wasReconstructed = NULL, GBool reconstruct = false);
 
   // Destructor.
   ~XRef();
 
+  // Copy xref but with new base stream!
+  XRef *copy();
+
   // Is xref table valid?
   GBool isOk() { return ok; }
 
@@ -90,6 +120,10 @@ public:
 		     Guchar *fileKeyA, int keyLengthA,
 		     int encVersionA, int encRevisionA,
 		     CryptAlgorithm encAlgorithmA);
+  // Mark Encrypt entry as Unencrypted
+  void markUnencrypted();
+
+  void getEncryptionParameters(Guchar **fileKeyA, CryptAlgorithm *encAlgorithmA, int *keyLengthA);
 
   // Is the file encrypted?
   GBool isEncrypted() { return encrypted; }
@@ -106,7 +140,7 @@ public:
   int getPermFlags() { return permFlags; }
 
   // Get catalog object.
-  Object *getCatalog(Object *obj) { return fetch(rootNum, rootGen, obj); }
+  Object *getCatalog(Object *obj);
 
   // Fetch an indirect reference.
   Object *fetch(int num, int gen, Object *obj, int recursion = 0);
@@ -124,10 +158,18 @@ public:
 
   // Get end position for a stream in a damaged file.
   // Returns false if unknown or file is not damaged.
-  GBool getStreamEnd(Guint streamStart, Guint *streamEnd);
+  GBool getStreamEnd(Goffset streamStart, Goffset *streamEnd);
 
   // Retuns the entry that belongs to the offset
-  int getNumEntry(Guint offset);
+  int getNumEntry(Goffset offset);
+
+  // Scans the document and sets special flags in all xref entries. One of those
+  // flags is Unencrypted, which affects how the object is fetched. Therefore,
+  // this function must be called before fetching unencrypted objects (e.g.
+  // Encrypt dictionary, XRef streams). Note that the code that initializes
+  // decryption doesn't need to call this function, because it runs before
+  // decryption is enabled, and therefore the Unencrypted flag is ignored.
+  void scanSpecialFlags();
 
   // Direct access.
   XRefEntry *getEntry(int i, GBool complainIfMissing = gTrue);
@@ -137,17 +179,21 @@ public:
   void setModifiedObject(Object* o, Ref r);
   Ref addIndirectObject (Object* o);
   void removeIndirectObject(Ref r);
-  void add(int num, int gen,  Guint offs, GBool used);
+  void add(int num, int gen,  Goffset offs, GBool used);
 
   // Output XRef table to stream
   void writeTableToFile(OutStream* outStr, GBool writeAllEntries);
   // Output XRef stream contents to GooString and fill trailerDict fields accordingly
   void writeStreamToBuffer(GooString *stmBuf, Dict *xrefDict, XRef *xref);
 
+  // to be thread safe during write where changes are not allowed
+  void lock();
+  void unlock();
+
 private:
 
   BaseStream *str;		// input stream
-  Guint start;			// offset in file (to allow for garbage
+  Goffset start;		// offset in file (to allow for garbage
 				//   at beginning of file)
   XRefEntry *entries;		// xref entries
   int capacity;			// size of <entries> array
@@ -155,8 +201,9 @@ private:
   int rootNum, rootGen;		// catalog dict
   GBool ok;			// true if xref table is valid
   int errCode;			// error code (if <ok> is false)
+  GBool xrefReconstructed;	// marker, true if xref was already reconstructed
   Object trailerDict;		// trailer dictionary
-  Guint *streamEnds;		// 'endstream' positions - only used in
+  Goffset *streamEnds;		// 'endstream' positions - only used in
 				//   damaged files
   int streamEndsLen;		// number of valid entries in streamEnds
   PopplerCache *objStrs;	// cached object streams
@@ -168,45 +215,65 @@ private:
   int permFlags;		// permission bits
   Guchar fileKey[32];		// file decryption key
   GBool ownerPasswordOk;	// true if owner password is correct
-  Guint prevXRefOffset;		// position of prev XRef section (= next to read)
-  Guint mainXRefEntriesOffset;	// offset of entries in main XRef table
+  Goffset prevXRefOffset;		// position of prev XRef section (= next to read)
+  Goffset mainXRefEntriesOffset; // offset of entries in main XRef table
   GBool xRefStream;		// true if last XRef section is a stream
+  Goffset mainXRefOffset;	// position of the main XRef table/stream
+  GBool scannedSpecialFlags;	// true if scanSpecialFlags has been called
+  GBool strOwner;     // true if str is owned by the instance
+#if MULTITHREADED
+  GooMutex mutex;
+#endif
 
   void init();
   int reserve(int newSize);
   int resize(int newSize);
-  Guint getStartXref();
-  GBool readXRef(Guint *pos, std::vector<Guint> *followedXRefStm);
-  GBool readXRefTable(Parser *parser, Guint *pos, std::vector<Guint> *followedXRefStm);
+  GBool readXRef(Goffset *pos, std::vector<Goffset> *followedXRefStm, std::vector<int> *xrefStreamObjsNum);
+  GBool readXRefTable(Parser *parser, Goffset *pos, std::vector<Goffset> *followedXRefStm, std::vector<int> *xrefStreamObjsNum);
   GBool readXRefStreamSection(Stream *xrefStr, int *w, int first, int n);
-  GBool readXRefStream(Stream *xrefStr, Guint *pos);
-  GBool constructXRef(GBool *wasReconstructed);
-  GBool parseEntry(Guint offset, XRefEntry *entry);
+  GBool readXRefStream(Stream *xrefStr, Goffset *pos);
+  GBool constructXRef(GBool *wasReconstructed, GBool needCatalogDict = gFalse);
+  GBool parseEntry(Goffset offset, XRefEntry *entry);
+  void readXRefUntil(int untilEntryNum, std::vector<int> *xrefStreamObjsNum = NULL);
+  void markUnencrypted(Object *obj);
 
   class XRefWriter {
   public:
     virtual void startSection(int first, int count) = 0;
-    virtual void writeEntry(Guint offset, int gen, XRefEntryType type) = 0;
+    virtual void writeEntry(Goffset offset, int gen, XRefEntryType type) = 0;
     virtual ~XRefWriter() {};
   };
 
+  // XRefWriter subclass that writes a XRef table
   class XRefTableWriter: public XRefWriter {
   public:
     XRefTableWriter(OutStream* outStrA);
     void startSection(int first, int count);
-    void writeEntry(Guint offset, int gen, XRefEntryType type);
+    void writeEntry(Goffset offset, int gen, XRefEntryType type);
   private:
     OutStream* outStr;
   };
 
+  // XRefWriter subclass that writes a XRef stream
   class XRefStreamWriter: public XRefWriter {
   public:
-    XRefStreamWriter(Object *index, GooString *stmBuf);
+    XRefStreamWriter(Object *index, GooString *stmBuf, int offsetSize);
     void startSection(int first, int count);
-    void writeEntry(Guint offset, int gen, XRefEntryType type);
+    void writeEntry(Goffset offset, int gen, XRefEntryType type);
   private:
     Object *index;
     GooString *stmBuf;
+    int offsetSize;
+  };
+
+  // Dummy XRefWriter subclass that only checks if all offsets fit in 4 bytes
+  class XRefPreScanWriter: public XRefWriter {
+  public:
+    XRefPreScanWriter();
+    void startSection(int first, int count);
+    void writeEntry(Goffset offset, int gen, XRefEntryType type);
+
+    GBool hasOffsetsBeyond4GB;
   };
 
   void writeXRef(XRefWriter *writer, GBool writeAllEntries);
