@@ -1,5 +1,5 @@
 /* poppler-link.cc: qt interface to poppler
- * Copyright (C) 2006-2007, Albert Astals Cid
+ * Copyright (C) 2006-2007, 2016, Albert Astals Cid
  * Copyright (C) 2007-2008, Pino Toscano <pino@kde.org>
  * Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
  * Copyright (C) 2012, Tobias Koenig <tokoe@kdab.com>
@@ -168,16 +168,40 @@ class LinkSoundPrivate : public LinkPrivate
 class LinkRenditionPrivate : public LinkPrivate
 {
 	public:
-		LinkRenditionPrivate( const QRectF &area, ::MediaRendition *rendition );
+		LinkRenditionPrivate( const QRectF &area, ::MediaRendition *rendition, ::LinkRendition::RenditionOperation operation, const QString &script, const Ref &annotationReference );
 		~LinkRenditionPrivate();
 
 		MediaRendition *rendition;
+		LinkRendition::RenditionAction action;
+		QString script;
+		Ref annotationReference;
 };
 
-	LinkRenditionPrivate::LinkRenditionPrivate( const QRectF &area, ::MediaRendition *r )
+	LinkRenditionPrivate::LinkRenditionPrivate( const QRectF &area, ::MediaRendition *r, ::LinkRendition::RenditionOperation operation, const QString &javaScript, const Ref &ref )
 		: LinkPrivate( area )
-		, rendition( new MediaRendition( r ) )
+		, rendition( r ? new MediaRendition( r ) : 0 )
+		, action( LinkRendition::PlayRendition )
+		, script( javaScript )
+		, annotationReference( ref )
 	{
+		switch ( operation )
+		{
+			case ::LinkRendition::NoRendition:
+				action = LinkRendition::NoRendition;
+				break;
+			case ::LinkRendition::PlayRendition:
+				action = LinkRendition::PlayRendition;
+				break;
+			case ::LinkRendition::StopRendition:
+				action = LinkRendition::StopRendition;
+				break;
+			case ::LinkRendition::PauseRendition:
+				action = LinkRendition::PauseRendition;
+				break;
+			case ::LinkRendition::ResumeRendition:
+				action = LinkRendition::ResumeRendition;
+				break;
+		}
 	}
 
 	LinkRenditionPrivate::~LinkRenditionPrivate()
@@ -265,21 +289,23 @@ class LinkMoviePrivate : public LinkPrivate
 		d->changeZoom = ld->getChangeZoom();
 		
 		int leftAux = 0, topAux = 0, rightAux = 0, bottomAux = 0;
-		
-		::Page *page;
-		if (d->pageNum > 0 &&
-		    d->pageNum <= data.doc->doc->getNumPages() &&
-		    (page = data.doc->doc->getPage( d->pageNum )))
-		{
-			cvtUserToDev( page, left, top, &leftAux, &topAux );
-			cvtUserToDev( page, right, bottom, &rightAux, &bottomAux );
-			
-			d->left = leftAux / (double)page->getCropWidth();
-			d->top = topAux / (double)page->getCropHeight();
-			d->right = rightAux/ (double)page->getCropWidth();
-			d->bottom = bottomAux / (double)page->getCropHeight();
+
+		if (!data.externalDest) {
+			::Page *page;
+			if (d->pageNum > 0 &&
+				d->pageNum <= data.doc->doc->getNumPages() &&
+				(page = data.doc->doc->getPage( d->pageNum )))
+			{
+				cvtUserToDev( page, left, top, &leftAux, &topAux );
+				cvtUserToDev( page, right, bottom, &rightAux, &bottomAux );
+
+				d->left = leftAux / (double)page->getCropWidth();
+				d->top = topAux / (double)page->getCropHeight();
+				d->right = rightAux/ (double)page->getCropWidth();
+				d->bottom = bottomAux / (double)page->getCropHeight();
+			}
+			else d->pageNum = 0;
 		}
-		else d->pageNum = 0;
 		
 		if (deleteDest) delete ld;
 	}
@@ -579,10 +605,15 @@ class LinkMoviePrivate : public LinkPrivate
 
 	// LinkRendition
 	LinkRendition::LinkRendition( const QRectF &linkArea, ::MediaRendition *rendition )
-		: Link( *new LinkRenditionPrivate( linkArea, rendition ) )
+		: Link( *new LinkRenditionPrivate( linkArea, rendition, ::LinkRendition::NoRendition, QString(), Ref() ) )
 	{
 	}
 	
+	LinkRendition::LinkRendition( const QRectF &linkArea, ::MediaRendition *rendition, int operation, const QString &script, const Ref &annotationReference )
+		: Link( *new LinkRenditionPrivate( linkArea, rendition, static_cast<enum ::LinkRendition::RenditionOperation>(operation), script, annotationReference ) )
+	{
+	}
+
 	LinkRendition::~LinkRendition()
 	{
 	}
@@ -598,6 +629,29 @@ class LinkMoviePrivate : public LinkPrivate
 		return d->rendition;
 	}
 
+	LinkRendition::RenditionAction LinkRendition::action() const
+	{
+		Q_D( const LinkRendition );
+		return d->action;
+	}
+
+	QString LinkRendition::script() const
+	{
+		Q_D( const LinkRendition );
+		return d->script;
+	}
+
+	bool LinkRendition::isReferencedAnnotation( const ScreenAnnotation *annotation ) const
+	{
+		Q_D( const LinkRendition );
+		if ( d->annotationReference.num != -1 && d->annotationReference == annotation->d_ptr->pdfObjectReference() )
+		{
+			return true;
+		}
+
+		return false;
+	}
+
 	// LinkJavaScript
 	LinkJavaScript::LinkJavaScript( const QRectF &linkArea, const QString &js )
 		: Link( *new LinkJavaScriptPrivate( linkArea ) )
