@@ -18,7 +18,7 @@
 // Copyright (C) 2007-2010, 2012 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
 // Copyright (C) 2011 Vittal Aithal <vittal.aithal@cognidox.com>
-// Copyright (C) 2012, 2013 Adrian Johnson <ajohnson@redneon.com>
+// Copyright (C) 2012, 2013, 2016 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2012 Fabio D'Urso <fabiodurso@hotmail.it>
 // Copyright (C) 2013 Adrian Perez de Castro <aperez@igalia.com>
 // Copyright (C) 2013 Suzuki Toshiya <mpsuzuki@hiroshima-u.ac.jp>
@@ -40,6 +40,7 @@
 #include "printencodings.h"
 #include "goo/GooString.h"
 #include "goo/gfile.h"
+#include "goo/glibc.h"
 #include "goo/gmem.h"
 #include "GlobalParams.h"
 #include "Object.h"
@@ -63,6 +64,7 @@
 static void printInfoString(Dict *infoDict, const char *key, const char *text,
 			    UnicodeMap *uMap);
 static void printInfoDate(Dict *infoDict, const char *key, const char *text);
+static void printISODate(Dict *infoDict, const char *key, const char *text);
 static void printBox(const char *text, PDFRectangle *box);
 static void printStruct(const StructElement *element, unsigned indent = 0);
 static void printIndent(unsigned level);
@@ -72,6 +74,7 @@ static int lastPage = 0;
 static GBool printBoxes = gFalse;
 static GBool printMetadata = gFalse;
 static GBool printJS = gFalse;
+static GBool isoDates = gFalse;
 static GBool rawDates = gFalse;
 static char textEncName[128] = "";
 static char ownerPassword[33] = "\001";
@@ -97,6 +100,8 @@ static const ArgDesc argDesc[] = {
    "print the logical document structure (for tagged files)"},
   {"-struct-text", argFlag, &printStructureText, 0,
    "print text contents along with document structure (for tagged files)"},
+  {"-isodates", argFlag,   &isoDates,         0,
+   "print the dates in ISO-8601 format"},
   {"-rawdates", argFlag,   &rawDates,         0,
    "print the undecoded date strings directly from the PDF file"},
   {"-enc",    argString,   textEncName,    sizeof(textEncName),
@@ -238,7 +243,10 @@ int main(int argc, char *argv[]) {
     printInfoString(info.getDict(), "Author",       "Author:         ", uMap);
     printInfoString(info.getDict(), "Creator",      "Creator:        ", uMap);
     printInfoString(info.getDict(), "Producer",     "Producer:       ", uMap);
-    if (rawDates) {
+    if (isoDates) {
+      printISODate(info.getDict(),   "CreationDate", "CreationDate:   ");
+      printISODate(info.getDict(),   "ModDate",      "ModDate:        ");
+    } else if (rawDates) {
       printInfoString(info.getDict(), "CreationDate", "CreationDate:   ",
 		      uMap);
       printInfoString(info.getDict(), "ModDate",      "ModDate:        ",
@@ -458,6 +466,7 @@ static void printInfoString(Dict *infoDict, const char *key, const char *text,
       n = uMap->mapUnicode(u[i], buf, sizeof(buf));
       fwrite(buf, 1, n, stdout);
     }
+    gfree(u);
     fputc('\n', stdout);
   }
   obj.free();
@@ -469,6 +478,7 @@ static void printInfoDate(Dict *infoDict, const char *key, const char *text) {
   int year, mon, day, hour, min, sec, tz_hour, tz_minute;
   char tz;
   struct tm tmStruct;
+  time_t time;
   char buf[256];
 
   if (infoDict->lookup(key, &obj)->isString()) {
@@ -486,8 +496,14 @@ static void printInfoDate(Dict *infoDict, const char *key, const char *text) {
       tmStruct.tm_yday = -1;
       tmStruct.tm_isdst = -1;
       // compute the tm_wday and tm_yday fields
-      if (mktime(&tmStruct) != (time_t)-1 &&
-	  strftime(buf, sizeof(buf), "%c", &tmStruct)) {
+      time = timegm(&tmStruct);
+      if (time != (time_t)-1) {
+	int offset = (tz_hour*60 + tz_minute)*60;
+	if (tz == '-')
+	  offset *= -1;
+	time -= offset;
+	localtime_r(&time, &tmStruct);
+	strftime(buf, sizeof(buf), "%c %Z", &tmStruct);
 	fputs(buf, stdout);
       } else {
 	fputs(s, stdout);
@@ -500,6 +516,33 @@ static void printInfoDate(Dict *infoDict, const char *key, const char *text) {
   obj.free();
 }
 
+void printISODate(Dict *infoDict, const char *key, const char *text)
+{
+  Object obj;
+  char *s;
+  int year, mon, day, hour, min, sec, tz_hour, tz_minute;
+  char tz;
+
+  if (infoDict->lookup(key, &obj)->isString()) {
+    fputs(text, stdout);
+    s = obj.getString()->getCString();
+    if ( parseDateString( s, &year, &mon, &day, &hour, &min, &sec, &tz, &tz_hour, &tz_minute ) ) {
+      fprintf(stdout, "%04d-%02d-%02dT%02d:%02d:%02d", year, mon, day, hour, min, sec);
+      if (tz_hour == 0 && tz_minute == 0) {
+	fprintf(stdout, "Z");
+      } else {
+	fprintf(stdout, "%c%02d", tz, tz_hour);
+	if (tz_minute)
+	  fprintf(stdout, ":%02d", tz_minute);
+      }
+    } else {
+      fputs(s, stdout);
+    }
+    fputc('\n', stdout);
+  }
+  obj.free();
+}
+
 static void printBox(const char *text, PDFRectangle *box) {
   printf("%s%8.2f %8.2f %8.2f %8.2f\n",
 	 text, box->x1, box->y1, box->x2, box->y2);
